# pygsear
# Copyright (C) 2003 Lee Harr
#
#
# This file is NOT part of pygsear.
# This is where I put the pygsear stuff that I had to change or sub-class.
# I feel un-pythonic. o.O

"""Graphical input devices"""


import time, random, math, os, sys, types
import colorsys
from code import InteractiveConsole
from code import compile_command

import pygame
import pygame.draw
from pygame.locals import K_RETURN, K_ESCAPE, K_BACKSPACE, K_F1, K_UP, K_DOWN
from pygame.locals import K_PAGEUP, K_PAGEDOWN, K_LEFT, K_RIGHT, K_DELETE
from pygame.locals import QUIT, MOUSEBUTTONUP


from pygsear import Drawable, conf, Path, Event, Util, locals

MOVING=0

#import conf
#import Drawable
#from Drawable import Rectangle
#import Path
#import Event
#import Util
#from locals import TRANSPARENT, BLACK, WHITE, LGREEN, LGRAY, GRAY, BLUE, RED

class Widget:
    def __init__(self, callback=None, group=()):
        self.set_callback(callback)
        self.events = Event.EventGroup()

    def set_callback(self, callback):
        if callback is None:
            callback = self.nop
        self.callback = callback

    def nop(self, arg=None, kwargs=None):
        pass

    def activate(self):
        self.active = 1

    def deactivate(self):
        self.active = 0

    def _stop(self, pygame_event=None):
        self.stop = 1

    def _quit(self, pygame_event=None):
        ev = pygame.event.Event(QUIT)
        pygame.event.post(ev)
        self._stop()

    def modal(self):
        stop = Event.KEYUP_Event(key=K_ESCAPE, callback=self._stop)
        while not self.stop:
            self.events.check()


class Button(Widget):
    def __init__(self, callback=None, group=None,kwargs=None):
        Widget.__init__(self)
        self.set_callback(callback)
        #print 'offset', offset, callback
        self.armed = 0
        self.events.add(Event.MOUSEBUTTONDOWN_Event(callback=self.clicked,kwargs=kwargs))
        self.events.add(Event.MOUSEBUTTONUP_Event(callback=self.released,kwargs=kwargs))
        if group is not None:
            group.add(self.events)
        self.stop = 0
        self.kwargs=kwargs

    def arm(self):
        self.armed = 1

    def fire(self, pygameEvent,kwargs):
        self.armed = 0
        self.callback(pygameEvent, self.kwargs)

    def clicked(self, pygameEvent,kwargs):
        pos = pygameEvent.pos
        try:
            offset = self.window.rect[0:2]
            #print 'off', offset
        except AttributeError:
            offset = (0, 0)

        if self.rect.move(offset[0], offset[1]).collidepoint(pos):
            self.arm()
            return 1
        else:
            return 0

    def released(self, pygameEvent, kwargs):
        #print 'rel'
        pos = pygameEvent.pos
        try:
            offset = self.window.rect[0:2]
            #print 'off', offset
        except AttributeError:
            offset = (0, 0)

        if self.rect.move(offset[0], offset[1]).collidepoint(pos) and self.armed:
            self.fire(pygameEvent,kwargs)
        else:
            self.armed = 0


class SpriteButton(Button, Drawable.Drawable):
    """Clickable button which is also a sprite."""

    def __init__(self, sprite, callback=None, group=None):
        """Initialize the button.
        """
        pos = sprite.get_position()
        Button.__init__(self, callback, group)
        Drawable.Drawable.__init__(self, w=sprite.window)
        self.image = sprite.image
        self.rect = sprite.rect
        self.set_path(sprite.path)

    def set_callback(self, callback,kwargs=None):
        if callback is None:
            callback = self.nop
        self.callback = callback
        self.kwargs=kwargs


class ImageButton(SpriteButton):
    filename = None

    def __init__(self, filename=None, callback=None, group=None):
        if filename is None:
            filename = self.filename
        sprite = Drawable.Image(filename=filename)
        SpriteButton.__init__(self, sprite=sprite, callback=callback, group=group)


class FlipButton(Drawable.Stationary, SpriteButton):
    def __init__(self,
                    window=None,
                    sprites=None,
                    callback=None,
                    group=None):
        pos = sprites[0].get_position()
        self.sprites=sprites
        Drawable.Stationary.__init__(self, w=window, sprite=sprites[0])
        SpriteButton.__init__(self, sprites[0], callback, group)
        self.image = sprites[0].image
        self.kwargs=None
        #self.rect = sprites[0].rect.inflate(-2,-2)
        self.set_position(pos)

    def clicked(self, pygameEvent=None,kwargs=None):
        pos = pygameEvent.pos
        try:
            offset = self.window.rect[0:2]
            #print 'off', offset
        except AttributeError:
            offset = (0, 0)

        if self.rect.move(offset[0], offset[1]).collidepoint(pos):
            self.image=self.sprites[1].image
            #self.sprites[1].draw()
            self.arm()
            return 1
        else:
            return 0

    def released(self, pygameEvent=None, kwargs=None):
        #print 'rel'
        pos = pygameEvent.pos
        try:
            offset = self.window.rect[0:2]
            #print 'off', offset
        except AttributeError:
            offset = (0, 0)

        if self.rect.move(offset[0], offset[1]).collidepoint(pos) and self.armed:
            self.image=self.sprites[0].image
            self.fire(pygameEvent,kwargs)
        else:
            self.armed = 0


class AnimButton(SpriteButton):
    def __init__(self, sprite, images):
        self.images=images
        self.kwargs=None
        self.flipticks = 20
        self.ticks = 20
        self.flipping = 0
        self.moving=0
        self.imageNum = 0
        SpriteButton.__init__(self,sprite=sprite,callback=None,group=None)
        self.rect.inflate(-3,-3)

    def flip(self):
        """switch to next image in sequence"""
        self.image = self.images[self.imageNum]
        self.imageNum += 1
        global MOVING
        MOVING=1
        self.flipping = 1

        if self.imageNum >= len(self.images):
            self.imageNum = 0
            self.flipping = 0
            MOVING=0
            if self.kind=='w':
                self.images=self.images_tb
            else:
                self.images=self.images_tw
        self.set_size(self.image.get_size())
        #self.set_crect()

    def animate(self):
        """set position to next position on path and flip to next image"""
        if self.flipping:
            self.ticks -= conf.ticks
            if self.ticks < 0:
                global MOVING
                MOVING=0
                self.flip()
                self.ticks += self.flipticks

    def move(self):
        """set position to next position on path"""
        global MOVING
        if self.moving:
            try:
                MOVING=1
                self._set_position(self.path.next())
            except StopIteration:
                MOVING=0
                self.moving=0
        if self.flipping:
            MOVING=1
            self.animate()

    def set_vel(self, whereto):
        """slides sprite 32 pixels to whereto"""
        speed=3
        start = self.get_position()
        destcoord=None
        if whereto=='up':
            dest = (start[0],start[1]-32)
            self.coord=(self.coord[0]-1, self.coord[1])
        elif whereto=='down':
            dest = (start[0],start[1]+32)
            self.coord=(self.coord[0]+1, self.coord[1])
        elif whereto=='left':
            dest = (start[0]-32, start[1])
            self.coord=(self.coord[0], self.coord[1]-1)
        else:
            dest = (start[0]+32, start[1])
            self.coord=(self.coord[0], self.coord[1]+1)

        self.path=Path.LinePath(startLocation=start,
                                endLocation=dest,
                                steps=speed)
        self.moving=1
        global MOVING
        MOVING=1
        return self.coord
