# classes and utilities that handle level loading

import data
from pygsear.Drawable import Image
from pygsear.Path import LinePath
from pygsear.Util import load_images
from MyWidget import SpriteButton, AnimButton

from resources import FILES_toblack, FILES_towhite

# self.map is the solved map. Bad naming.
# self.unsolved_map is the actual game map you see on screen.

class Level:
    """The whole game board
    """
    def __init__(self, filename=None):
        try:
            f = open(data.filepath('levels/'+filename),'r')
            self.difficulty = int(f.readline().strip())
            self.start_color = f.readline().strip()
            map = []
            unsolved_map = []
            self.wcount=0
            self.bcount=0
            for i in f:
                line = []
                unsolved_line=[]
                for j in  i.strip():
                    line.append(j)
                    if j!='g':
                        unsolved_line.append('w')
                        if j=='w':
                            self.wcount+=1
                        else:
                            self.bcount+=1
                    else:
                        unsolved_line.append('g')
                map.append(line)
                unsolved_map.append(unsolved_line)
            self.map = map
            self.unsolved_map=unsolved_map
            f.close()
        except:  #set dummy level
            print 'Loading dummy level..'
            self.start_color='w'
            self.difficulty = 0
            self.free=(4,4)
            self.map = [['w','w','w','w','w','w'],
                        ['w','w','w','w','w','w'],
                        ['w','w','b','b','w','w'],
                        ['w','w','b','b','w','w'],
                        ['w','w','w','w','w','w'],
                        ['w','w','w','w','w','g']]
            self.unsolved_map = []
            self.bcount=4
            self.wcount=31
            for i in self.map:
                line=[]
                for j in i:
                    if j!='g':
                        line.append('w')
                    else:
                        line.append('g')
                self.unsolved_map.append(line)
        pieces=[]
        row=0
        for i in self.unsolved_map:
            line=[]
            col=0
            for j in i:
                if j!='g':
                    line.append(Piece(j,(row,col)))
                col+=1
            row+=1
            pieces.append(line)

        self.free=(row-1,col-1)
        self.size=(row-1,col-1)
        self.pieces=pieces

    def reinit(self):
        self.unsolved_map=[]
        for i in self.pieces:
            unsolved_line=[]
            for j in i:
                unsolved_line.append('w')
            self.unsolved_map.append(unsolved_line)
        self.unsolved_map[-1].append('g')

        self.pieces=[]
        row=0
        for i in self.unsolved_map:
            line=[]
            col=0
            for j in i:
                if j!='g':
                    line.append(Piece(j,(row,col)))
                col+=1
            row+=1
            self.pieces.append(line)
        self.free=(row-1,col-1)

    def get_piece(self, coord):
        """returns piece that is in the solved map position"""
        return self.pieces[coord[0]][coord[1]]

    def get_piece_at(self, coord):
        """returns piece at specified coordinates"""
        for line in self.pieces:
            for piece in line:
                if piece.coord==coord:
                    return piece
        #print "Could not find piece at ", coord
        return None

    def change_piece(self, coord, kind):
        p=self.get_piece_at((coord[0],coord[1]))
        p.change(kind)
        self.unsolved_map[coord[0]][coord[1]]=kind

    def move_piece(self, coord, whereto):
        p=self.get_piece_at((coord[0],coord[1]))
        if p:
            dest=p.set_vel(whereto)
            self.unsolved_map[dest[0]][dest[1]]=p.kind
            self.unsolved_map[coord[0]][coord[1]]='g'

class Piece(AnimButton):
    """The black/white game piece/s"""
    def __init__(self, kind, coord):
        self.moving=0
        self.flipping=0
        self.images_tb=load_images(FILES_toblack)
        self.images_tw=load_images(FILES_towhite)
        if kind=='w':
            self.images=self.images_tb
        else:
            self.images=self.images_tw
        self.sprite=Image(image=self.images[0])
        AnimButton.__init__(self, self.sprite, self.images)
        self.set_position((-32,-32))
        self.kind=kind
        self.coord=coord

    def change(self, kind):
        self.kind=kind
        pos = self.get_position()
        self.set_position(pos)
        self.flipping=1

    def reinit(self, coord):
        self.kind='w'
        self.imageNum = 0
        self.images=self.images_tb
        self.image=self.images[0]
        self.coord=coord
        self.set_position((-32,-32))

if __name__=='__main__':
    m = Level()
    #print m.difficulty
    #print m.start_color
    print 'Solved'
    for i in m.map:
        print i
    print 'Unsolved'
    for i in m.unsolved_map:
        print i

    p=m.get_piece_at((4,5))
    print p.coord
    m.change_piece((4,5),'b')
    m.move_piece((4,5),'down')

    print m.free
    for i in m.unsolved_map:
        print i
    print
    print p.coord
    m.reinit()
    for i in m.unsolved_map:
        print i
    print p.coord
